from plico_motor.client.snapshot_entry import SnapshotEntry


class MotorStatus(object):

    TYPE_LINEAR = 'linear'
    TYPE_ROTATIVE = 'rotative'

    def __init__(self,
                 name,
                 position,
                 steps_per_SI_unit,
                 was_homed,
                 motor_type,
                 is_moving,
                 last_commanded_position
                 ):
        self.name = name
        self.position = position
        self.steps_per_SI_unit = steps_per_SI_unit
        self.was_homed = was_homed
        self.motor_type = motor_type
        self.is_moving = is_moving
        self.last_commanded_position = last_commanded_position

    def as_dict(self):
        dicto = {}
        dicto[SnapshotEntry.MOTOR_NAME] = self.name
        dicto[SnapshotEntry.POSITION] = self.position
        dicto[SnapshotEntry.STEPS_PER_SI_UNITS] = self.steps_per_SI_unit
        dicto[SnapshotEntry.WAS_HOMED] = self.was_homed
        dicto[SnapshotEntry.MOTOR_TYPE] = self.motor_type
        dicto[SnapshotEntry.IS_MOVING] = self.is_moving
        dicto[SnapshotEntry.LAST_COMMANDED_POSITION] = self.last_commanded_position
        return dicto






class FakeTimeMod(object):

    def __init__(self, timeInvocationDurationSec=0.05):
        self._currentTime = 0.0
        self._timeInvocationDurationSec = timeInvocationDurationSec
        self._lastSleepDurationSec = None

    def time(self):
        self._currentTime += self._timeInvocationDurationSec
        return self._currentTime

    def sleep(self, sleepDurationSec):
        self._lastSleepDurationSec = sleepDurationSec
        self._currentTime += sleepDurationSec

    def hasSleepBeenInvoked(self):
        return self._lastSleepDurationSec is not None

    def getLastSleepDurationSec(self):
        return self._lastSleepDurationSec



import sys

import plico_motor
from guietta import Gui, _


class Runner(object):

    def __init__(self):
        pass

    def _setUp(self, argv):

        host, port = argv
        self._motor = plico_motor.motor(host, int(port))
        print(dir(self._motor))

        def moveby(gui):
            nsteps = int(gui.nstepsby)
            self._motor.move_by(nsteps)

        def moveto(gui):
            nsteps = int(gui.nstepsto)
            self._motor.move_to(nsteps)

        def getstatus(gui):
            try:
                gui.pos = self._motor.position()
                gui.status = self._motor.status().as_dict()
            except Exception as e:
                gui.pos = str(e)

        self.gui = Gui(
             [  'Pos:'     , 'pos'       , _       ],
             [ ['Move to'] , '__nstepsto__', 'steps' ],
             [ ['Move by'] , '__nstepsby__', 'steps' ],
             [ 'Status:'   , 'status'    , _       ]
        )
        self.gui.Moveby = moveby
        self.gui.Moveto = moveto
        self.gui.timer_start(getstatus, 0.1)

    def run(self, argv):
        self._setUp(argv)
        self.gui.run()

    def terminate(self, signal, frame):
        pass


if __name__ == '__main__':
    runner = Runner()
    sys.exit(runner.run(sys.argv[1:]))




import abc
from six import with_metaclass
from plico.utils.decorator import returnsNone, returns, returnsForExample
from plico_motor.types.motor_status import MotorStatus


class AbstractMotorClient(with_metaclass(abc.ABCMeta, object)):

    @abc.abstractmethod
    @returns(int)
    def position(self):
        assert False

    @abc.abstractmethod
    @returnsNone
    def move_to(self, position_in_steps):
        assert False

    @abc.abstractmethod
    @returnsNone
    def move_by(self, position_in_steps):
        assert False

    @abc.abstractmethod
    @returnsForExample({'MY_MOTOR.POS: 42'})
    def snapshot(self, prefix='MY_MOTOR'):
        assert False

    @abc.abstractmethod
    @returnsNone
    def home(self):
        assert False

    @abc.abstractmethod
    @returns(MotorStatus)
    def status(self):
        assert False

# def _proposed_interface():
#     x = plico_motor.client('ip', axis=0, port=port)
#     x.home()
#     x.home(limit=x.NEG)
#     curr_pos = x.position()
#     x.move_by(-23)
#     x.move_to(123)
#
#     # status. A given field can be MotorClient.NOT_AVAILABLE if the motor is
#     # not providing the info.
#     status = x.status()
#     assert status.has_been_homed
#     steps_per_m = status.steps_per_m
#     max_speed = status.max_speed
#     curr_pos = status.position
#     is_moving = status.is_moving
#     should_be_in = status.last_commanded_position
#
#     # snapshot is like status in a dictionary form ready for FITS headers
#     dicto = x.snapshot('MYAXIS1')
#     assert dicto['MYAXIS1.POS'] == 123
#     # and so on for each status' fields
#
#     # boh... nice to have. Need to modify config file on server side.
#     x.move_to('A')
#     x.save_current_position_as('A')
#     x.list_positions()
#
#     # boh... Is this needed in the client? Static conf should be enough.
#     x.set_max_speed(100)




import unittest
from plico_motor.types.motor_status import MotorStatus
from plico_motor.client.snapshot_entry import SnapshotEntry


class Test(unittest.TestCase):

    def setUp(self):
        self.ms = MotorStatus(
            'pippo',
            123,
            987654,
            False,
            MotorStatus.TYPE_ROTATIVE,
            True,
            333)

    def test_as_dict(self):
        wanted_keys = (
            SnapshotEntry.MOTOR_NAME,
            SnapshotEntry.POSITION,
            SnapshotEntry.STEPS_PER_SI_UNITS,
            SnapshotEntry.WAS_HOMED,
            SnapshotEntry.MOTOR_TYPE,
            SnapshotEntry.IS_MOVING,
            SnapshotEntry.LAST_COMMANDED_POSITION)
        got = self.ms.as_dict()
        self.assertCountEqual(got.keys(), wanted_keys)


if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



from plico_motor.utils.constants import Constants


def _getDefaultConfigFilePath():
    from plico.utils.config_file_manager import ConfigFileManager
    cfgFileMgr = ConfigFileManager(Constants.APP_NAME,
                                   Constants.APP_AUTHOR,
                                   Constants.THIS_PACKAGE)
    return cfgFileMgr.getConfigFilePath()


defaultConfigFilePath = _getDefaultConfigFilePath()


def motor(hostname, port):

    from plico_motor.client.motor_client import MotorClient
    from plico.rpc.zmq_remote_procedure_call import ZmqRemoteProcedureCall
    from plico.rpc.zmq_ports import ZmqPorts
    from plico.rpc.sockets import Sockets

    rpc = ZmqRemoteProcedureCall()
    zmqPorts = ZmqPorts(hostname, port)
    sockets = Sockets(zmqPorts, rpc)
    return MotorClient(rpc, sockets)



#!/usr/bin/env python
import sys
from plico_motor.gui.motor_control_gui import Runner


def main():
    runner = Runner()
    print("%s" % sys.argv)
    sys.exit(runner.run(sys.argv[1:]))


if __name__ == '__main__':
    main()




class SnapshotEntry(object):
    MOTOR_NAME = 'NAME'
    POSITION = "POS"
    STEPS_PER_SI_UNITS = 'SI2STEPS'
    WAS_HOMED = 'WASHOMED'
    MOTOR_TYPE = 'TYPE'
    IS_MOVING = 'ISMOVING'
    LAST_COMMANDED_POSITION = 'LASTCMD'



#!/usr/bin/env python
from plico_motor.client.abstract_motor_client import AbstractMotorClient
from plico.rpc.abstract_remote_procedure_call import \
    AbstractRemoteProcedureCall
from plico.utils.logger import Logger
from plico.utils.decorator import override, returns
from plico.utils.snapshotable import Snapshotable
from plico.client.serverinfo_client import ServerInfoClient
from plico.client.hackerable_client import HackerableClient
from plico_motor.types.motor_status import MotorStatus
from plico_motor.utils.timeout import Timeout


class MotorClient(AbstractMotorClient,
                  HackerableClient,
                  ServerInfoClient):

    def __init__(self,
                 rpcHandler,
                 sockets):
        assert isinstance(rpcHandler, AbstractRemoteProcedureCall)

        self._rpcHandler = rpcHandler
        self._requestSocket = sockets.serverRequest()
        self._statusSocket = sockets.serverStatus()
        self._logger = Logger.of('Motor client')
        HackerableClient.__init__(self,
                                  self._rpcHandler,
                                  self._requestSocket,
                                  self._logger)
        ServerInfoClient.__init__(self,
                                  self._rpcHandler,
                                  self._requestSocket,
                                  self._logger)

    @override
    @returns(MotorStatus)
    def status(self, timeout_in_sec=Timeout.GETTER):
        return self._rpcHandler.receivePickable(
            self._statusSocket,
            timeout_in_sec)

    @override
    def snapshot(self,
                 prefix,
                 timeout_in_sec=Timeout.GETTER):
        self._logger.notice("Getting snapshot for %s " % prefix)
        status = self.status(timeout_in_sec=timeout_in_sec)
        return Snapshotable.prepend(prefix, status.as_dict())

    @override
    def home(self,
             timeout_in_sec=Timeout.SETTER):
        self._logger.notice("Homing")
        return self._rpcHandler.sendRequest(
            self._requestSocket, 'home',
            [],
            timeout=timeout_in_sec)

    @override
    def move_to(self,
                position_in_steps,
                timeout_in_sec=Timeout.SETTER):
        self._logger.notice("Moving to %f" % position_in_steps)
        return self._rpcHandler.sendRequest(
            self._requestSocket, 'move_to',
            [position_in_steps],
            timeout=timeout_in_sec)

    @override
    def move_by(self,
                position_in_steps,
                timeout_in_sec=Timeout.SETTER):
        self._logger.notice("Moving by %f" % position_in_steps)
        return self._rpcHandler.sendRequest(
            self._requestSocket, 'move_by',
            [position_in_steps],
            timeout=timeout_in_sec)

    @override
    def position(self):
        return int(self.status().position)




from plico_motor.client.abstract_motor_client import AbstractMotorClient
from plico.utils.decorator import override, returns
from plico_motor.types.motor_status import MotorStatus
from plico.utils.snapshotable import Snapshotable


class SimulatedMotorClient(AbstractMotorClient):

    def __init__(self):
        self._name = 'mySimulatedMotor'
        self._position = 0
        self._was_homed = False
        self._step_per_si_units = 123456
        self._type = MotorStatus.TYPE_LINEAR
        self._is_moving = False
        self._last_cmd = 0

    @override
    def position(self):
        return self._position

    @override
    def move_to(self, position_in_steps):
        self._position = position_in_steps
        self._last_cmd = self._position

    @override
    def move_by(self, position_in_steps):
        self._position += position_in_steps
        self._last_cmd = self._position

    @override
    def home(self):
        self._position = 0
        self._was_homed = True
        self._last_cmd = 0

    @override
    def snapshot(self, prefix):
        status = self.status()
        return Snapshotable.prepend(prefix, status.as_dict())

    @override
    @returns(MotorStatus)
    def status(self):
        status = MotorStatus(
            self._name,
            self._position,
            self._was_been_homed,
            self._step_per_si_units,
            self._was_homed,
            self._type,
            self._is_moving,
            self._last_cmd
            )
        return status

